package com.dayou.controller;

import com.dayou.annotation.OperLog;
import com.dayou.bo.PrivilegeTreeBO;
import com.dayou.condition.LoginCondition;
import com.dayou.dto.LoginDTO;
import com.dayou.dto.UpdatePasswordDTO;
import com.dayou.dto.UserBaseDetailDTO;
import com.dayou.enums.OperationTypeEnum;
import com.dayou.vo.UserVO;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.service.IUserService;
import com.dayou.entity.User;
import com.dayou.common.RestResponse;
import org.springframework.web.bind.annotation.*;
import com.dayou.common.PullDownModel;
import com.dayou.utils.ConvertUtil;
import com.dayou.utils.HttpKit;
import com.dayou.exception.ErrorCode;

import java.util.List;
import java.util.Set;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;

import static com.dayou.constants.JwtConstants.REST_TOKEN;

/**
 * 基础用户
 *
 * @author wucl
 * @since 2022-11-29
 */
@RestController
@RequestMapping("user")
@Slf4j
public class UserController extends BaseController {
    @Autowired
    private IUserService userService;

    /**
    * 基础用户列表 部门
    */
    @GetMapping("")
    public RestResponse<Page<UserVO>> page(UserVO userVO, Page page){
        Page<UserVO> pages=userService.xPage(page,userVO);
        return RestResponse.data(pages);
    }

    /**
     * 基础用户详情
     */
    @GetMapping("/{id}")
    public RestResponse<UserVO> detail(@PathVariable Long id){
        UserVO xUser =userService.detail(id);
        return RestResponse.data(xUser);
     }

    /**
     * 基础用户新增
     */
    @OperLog(title = "用户新增")
    @PostMapping("")
    public RestResponse<Boolean> save(@RequestBody @Valid UserVO user) {
        Boolean ret = userService.xSave(user);
        return RestResponse.data(ret);
    }

    /**
     * 基础用户更新
     */
    @OperLog(title = "用户更新")
    @PutMapping("")
    public RestResponse<Boolean> update(@RequestBody @Valid UserVO user) {
        Boolean ret = userService.xUpdate(user);
        return RestResponse.data(ret);
    }

    /**
     * 基础用户删除
     */
    @OperLog(title = "用户删除")
    @DeleteMapping("/{id}")
    public RestResponse<Boolean> delete(@PathVariable Long id) {
        Boolean ret = userService.delete(id);
        return RestResponse.data(ret);
    }


    /**
     * 基础用户下拉列表
     */
    @GetMapping("/simpleAll")
    public RestResponse<List<PullDownModel>> simpleAll(){
        LambdaQueryWrapper<User> lambdaQueryWrapper = new LambdaQueryWrapper<User>()
        .select(User::getId,User::getName)
        .eq(User::getDeleted,false);
        List<User> list= userService.list(lambdaQueryWrapper);
        List<PullDownModel> ret=ConvertUtil.copyList(list, PullDownModel.class);
        return RestResponse.data(ret);
    }
    /**
     * 基础用户模板下载
     */
    @OperLog(title = "基础用户模板下载",businessType = OperationTypeEnum.EXPORT)
    @GetMapping(value = "/download/importTemplate")
    public void importTemplate() {
        try {
            XSSFWorkbook wb = userService.generateImportTemplate();

            String fileName = "基础用户导入模板" + System.currentTimeMillis() + ".xlsx";

            writeXss2Response(HttpKit.getRequest(), HttpKit.getResponse(), wb, fileName);
        } catch (Exception e) {
            log.error("生成基础用户导入模板异常", e);
            ErrorCode.throwBusinessException(ErrorCode.DEFAULTERROR);
        }
    }

    /**
     * 基础用户导入
     */
    @OperLog(title = "基础用户模板导入",businessType = OperationTypeEnum.IMPORT)
    @PostMapping("/importExcel")
    public RestResponse<Boolean> importExcel(@RequestPart("file") MultipartFile file) {
        userService.importExcel(file);
        return RestResponse.data(true);
    }

    /**
     * 基础用户导出
     */
    @GetMapping(value = "/download/exportExcel")
    public void export(User user) {
        try {
            XSSFWorkbook wb =  userService.exportExcel(user);

            String fileName = "基础用户" + System.currentTimeMillis() + ".xlsx";

            writeXss2Response(HttpKit.getRequest(), HttpKit.getResponse(), wb, fileName);
        } catch (Exception e) {
            log.error("导出基础用户", e);
            ErrorCode.throwBusinessException(ErrorCode.DEFAULTERROR);
        }

    }

    /**
     * 登录
     *
     * @param loginCondition
     * @return
     */
    @PostMapping("/login")
    @OperLog(title = "登录", businessType = OperationTypeEnum.LOGIN)
    public RestResponse<LoginDTO> login(@RequestBody @Valid LoginCondition loginCondition) {
        LoginDTO loginDTO = userService.login(loginCondition);
        return RestResponse.data(loginDTO);
    }

    /**
     * 根据token获取基本信息
     *
     * @return
     */
    @GetMapping("/baseInfo")
    public RestResponse<UserBaseDetailDTO> baseInfo() {
        String token = HttpKit.getHeaderValue(REST_TOKEN);
        UserBaseDetailDTO userBaseDetailDTO = userService.baseInfo(token);
        return RestResponse.data(userBaseDetailDTO);
    }

    /**
     * 根据token获取用户权限
     *
     * @return
     */
    @GetMapping("/privilegesTree")
    public RestResponse<Set<PrivilegeTreeBO>> privilegesTree() {
        String token = HttpKit.getHeaderValue(REST_TOKEN);
        Set<PrivilegeTreeBO> list = userService.privilegesTree(token);
        return RestResponse.data(list);
    }

    /**
     * 重置密码
     * @param id
     * @return
     */
    @OperLog(title = "重置密码",businessType = OperationTypeEnum.RESET)
    @PutMapping("/resetPassword/{id}")
    public RestResponse<Boolean> restPassword(@PathVariable Long id){
        Boolean result = userService.restPassword(id);
        return RestResponse.data(result);
    }

    /**
     * 修改密码
     * @param passwordDTO
     * @return
     */
    @OperLog(title = "修改密码",businessType = OperationTypeEnum.SET)
    @PutMapping("/updatePassword")
    public RestResponse<Boolean> updatePassword(@RequestBody UpdatePasswordDTO passwordDTO){
        Boolean result = userService.updatePassword(passwordDTO);
        return RestResponse.data(result);
    }
}

