package com.dayou.brokerage.handler;

import com.dayou.bo.BrokerageDetailBO;
import com.dayou.bo.LeaderRatioBO;
import com.dayou.brokerage.BrokerageCalculateSupport;
import com.dayou.brokerage.MarketerBrokerageCalculator;
import com.dayou.brokerage.constants.BrokerageMode;
import com.dayou.brokerage.constants.BrokerageRule;
import com.dayou.brokerage.constants.BrokerageState;
import com.dayou.entity.ItemBrokerageDetail;
import com.dayou.enums.GlobalConfigEnum;
import com.dayou.exception.BusinessException;
import com.google.common.cache.LoadingCache;
import com.google.common.collect.Sets;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Set;
import java.util.concurrent.ExecutionException;
import java.util.stream.Collectors;

/**
 * 类说明：土地规划部市场人员提成计算处理器
 *
 * @author: wucl
 * @since: 2023/3/6
 * created with IntelliJ IDEA.
 */
@Component
@Slf4j
public class LandMarketerBrokerageHandler extends BrokerageCalculateSupport implements MarketerBrokerageCalculator {


    @Autowired
    private LoadingCache<String,BigDecimal> marketerRatioCache;

    @Autowired
    private LoadingCache<String,BigDecimal> globalConfigCache;

    @Override
    public BrokerageDetailBO predictPersonalAmount(BrokerageDetailBO bo) {
        try {
            String mergeKey = bo.mergeKey();
            BigDecimal ratio = marketerRatioCache.get(mergeKey);
            bo.setRate(ratio);
            if (ratio !=null){
                checkItemBrokerageStatus(bo);
                bo.setPredictAmount(bo.getAmount().multiply(ratio));
                bo.setBrokerageMode(BrokerageMode.PERSONAL.getCode());
                bo.setActualAmount(actualAmount(bo.getPredictAmount()));
                bo.setMarketerAdvanceBrokerageAble(checkMarketerPredictBrokerage(bo));
                if (checkMarketerPredictBrokerage(bo)){
                    bo.setMarketerStatus(BrokerageState.TO_PREDICTING.getCode());
                    bo.setAheadAmount(aheadAmount(bo.getPredictAmount()));
                }
            }else {
                log.info("项目id:[{}],合同金额：[{}],提成系数：[null],[未设置提成系数，无法计算客户经理提成]",bo.getId(),bo.getAmount());
            }
        }   catch (BusinessException e) {
            log.info("项目id:[{}],客户经理id：[{}],[{}]",bo.getId(),bo.getUserId(),e.getMessage());
        } catch (ExecutionException e) {
            throw new RuntimeException(e);
        }
        return bo;
    }

    @Override
    public BigDecimal aheadAmount(BigDecimal predictAmount){
        try {
            BigDecimal aheadAmount = predictAmount.multiply(globalConfigCache.get(GlobalConfigEnum.Finance.MARKETER_BROKERAGE_PERCENTAGE.getCode()));
            return aheadAmount;
        } catch (ExecutionException e) {
            log.error("获取提成缓存数据失败");
            return BigDecimal.ZERO;
        }

    }

    @Override
    public Set<ItemBrokerageDetail> predictTeamShareAmount(BrokerageDetailBO bo, Set<LeaderRatioBO> leaderRatioBOSet) {
        Set<ItemBrokerageDetail> itemBrokerageDetails = Sets.newHashSet();
        try {
            String mergeKey = bo.mergeKey();
            BigDecimal ratio = marketerRatioCache.get(mergeKey);
            if (ratio!=null){
                checkItemBrokerageStatus(bo);
                itemBrokerageDetails = leaderRatioBOSet.stream().map(x -> {
                    ItemBrokerageDetail itemBrokerageDetail = new ItemBrokerageDetail();
                    itemBrokerageDetail.setGeneralId(bo.getGeneralId());
                    itemBrokerageDetail.setPredictAmount(bo.getAmount().multiply(ratio).multiply(x.getRatio()));
                    itemBrokerageDetail.setUserId(x.getUserId());
                    itemBrokerageDetail.setAheadAmount(BigDecimal.ZERO);
                    itemBrokerageDetail.setActualAmount(actualAmount(itemBrokerageDetail.getPredictAmount()));
                    itemBrokerageDetail.setAdvanceAmount(BigDecimal.ZERO);
                    itemBrokerageDetail.setBrokerageMode(BrokerageMode.TEAM_SHARE.getCode());
                    itemBrokerageDetail.setBrokerageRule(BrokerageRule.LAND_MARKETER_RULE.getCode());
                    return itemBrokerageDetail;
                }).collect(Collectors.toSet());

            }else {
                log.info("项目id:[{}],合同金额：[{}],提成系数：[null],[未设置提成系数，无法计算客户经理领导提成]",bo.getId(),bo.getAmount());
            }
        } catch (ExecutionException e) {
            log.error("获取提成缓存数据失败");
        }
        return itemBrokerageDetails;
    }

    @Override
    public BigDecimal actualAmount(BigDecimal predictAmount) {
        //客户经理的提成不存在考核，即 预计提成金额 == 实际提成金额
        return predictAmount;
    }

}
